from __future__ import print_function
import pytest
from voeventdb.server.tests.fixtures.connection import simple_populated_db

import voeventdb.remote.apiv1 as apiv1

def test_mock_requests_empty_db(mock_requests):
    count = apiv1.count()
    assert count == 0

def test_mock_requests_simpledb(mock_requests, simple_populated_db):
    dbinf = simple_populated_db
    count = apiv1.count()
    assert count == dbinf.n_inserts


from __future__ import print_function
import pytest
import voeventdb.remote as vr
import voeventdb.remote.request_wrappers as wrappers
from voeventdb.remote.definitions import ResultKeys
from voeventdb.remote.apiv1.definitions import Endpoints, FilterKeys
import requests
from voeventdb.server.tests.fixtures.connection import simple_populated_db
import voeventdb.server.tests.fixtures.fake as fake
import datetime
import iso8601
from astropy.coordinates import SkyCoord, Angle
import astropy.units as units


@pytest.mark.usefixtures('mock_requests', 'simple_populated_db')
class TestGetSummaryWraps():
    def test_default_params(self, simple_populated_db):
        dbinf = simple_populated_db
        count = wrappers.get_summary_data(endpoint=Endpoints.count,
                                          filters=None,
                                          host=vr.default_host)
        assert count == dbinf.n_inserts

    def test_list_wrapper_pagination(self, simple_populated_db,
                                     reset_globals_to_defaults):
        vr.default_list_n_max = 30
        vr.default_pagesize = 5
        dbinf = simple_populated_db
        count = wrappers.get_summary_data(endpoint=Endpoints.count,
                                          filters=None,
                                          host=vr.default_host)
        assert count > vr.default_list_n_max
        ivorns = wrappers.get_list_data(
            list_endpoint=Endpoints.list_ivorn,
            count_endpoint=Endpoints.count,
        )
        assert len(ivorns) == vr.default_list_n_max

        # This time we override the max:
        ivorns = wrappers.get_list_data(
            list_endpoint=Endpoints.list_ivorn,
            count_endpoint=Endpoints.count,
            n_max=0,
        )
        assert len(ivorns) == dbinf.n_inserts

    def test_filter_by_datetime(self, simple_populated_db):
        """
        Check default requests behaviour when passed datetime-params.
        """
        dbinf = simple_populated_db
        cutoff_time = fake.default_start_dt + datetime.timedelta(days=0.75)
        filters = {
            FilterKeys.authored_since: cutoff_time
        }

        qualifying_packets = [p for p in dbinf.insert_packets
                              if iso8601.parse_date(
                p.Who.Date.text) >= cutoff_time]

        # Grab a copy of the full response dict to check that datetimes
        # are acceptably formatted by requests:
        rv = wrappers.get_summary_response(endpoint=Endpoints.count,
                                           filters=filters,
                                           host=vr.default_host)
        rd = rv.json()
        dt_string = rd[ResultKeys.querystring][FilterKeys.authored_since][0]
        assert iso8601.parse_date(dt_string) == cutoff_time

        count = wrappers.get_summary_data(endpoint=Endpoints.count,
                                          filters=filters,
                                          host=vr.default_host)
        assert count != 0
        assert count < dbinf.n_inserts
        assert count == len(qualifying_packets)


class TestConeParamParsing():
    @pytest.fixture(autouse=True)
    def assign_test_values(self):
        self.posn = SkyCoord(180, 0, unit='deg')
        self.posn_err = Angle(5, unit='deg')
        self.cone = (self.posn, self.posn_err)
        self.cone_string = str(
            [self.posn.ra.deg, self.posn.dec.deg, self.posn_err.deg])

    def test_string_format_still_valid(self, mock_requests):
        rv = requests.get(url=vr.default_host + Endpoints.count,
                          params={FilterKeys.cone: self.cone_string})
        # print(rv.text)
        assert rv.status_code == 200

    def test_valid_string_passthrough(self):
        assert (wrappers.format_conesearch_params(self.cone_string) ==
                self.cone_string)

    def test_skycoord_angle_pair(self):
        input = (self.posn, self.posn_err)
        assert wrappers.format_conesearch_params(input) == self.cone_string

    def test_skycoord_units_pair(self):
        input = (self.posn, self.posn_err.deg * units.deg)
        assert wrappers.format_conesearch_params(input) == self.cone_string

    def test_float_tuple(self):
        input = (self.posn.ra.deg, self.posn.dec.deg, self.posn_err.deg)
        assert wrappers.format_conesearch_params(input) == self.cone_string



from __future__ import print_function
import pytest
import flask
from voeventdb.server.tests.fixtures.connection import (
    empty_db_connection,
    fixture_db_session,
    simple_populated_db
)
from voeventdb.server.restapi.app import app
import voeventdb.remote as vr
import requests
import logging

logger = logging.getLogger('vdbr-tests')


@pytest.yield_fixture
def mock_requests(fixture_db_session):
    # print
    # print "setting up flask client"
    app.testing = True
    client = app.test_client()
    app_context = app.test_request_context()
    app_context.push()

    def mock_requests_get(url, params=None, **kwargs):
        """
        A mocked version of requests.get, fetches using the flask test client.

        This is pretty ugly, but I think it makes sense now:
        We 'prepare' the request, which takes the 'params' and builds them
        into the url. We can extract the url minus the 'host' prefix as
        ``prepped.path_url``, which is fortunately exactly what the flask
        ``test_client.get()`` method needs as an argument.

        Now, we want all the usual requests.Response behaviours, so we
        manually initialise one and assign ``._content`` as the data returned
        from our flask test-client.
        """
        # print("USING MOCKED GET FUNCTION")
        req = requests.Request('GET', url, params=params, **kwargs)
        prepped = req.prepare()
        full_url = prepped.url
        # print("PATH URL", prepped.path_url)
        logger.debug("Fetching {}".format(prepped.path_url))
        flask_response = client.get(prepped.path_url)
        req_resp = requests.Response()
        req_resp._content = flask_response.data
        req_resp.status_code = flask_response.status_code
        return req_resp

    original_get = requests.get
    requests.get = mock_requests_get
    yield
    app_context.pop()
    del app_context
    requests.get = original_get


@pytest.yield_fixture
def reset_globals_to_defaults():
    host = vr.default_host
    list_max = vr.default_list_n_max
    pagesize = vr.default_pagesize
    yield
    vr.default_host = host
    vr.default_list_n_max = list_max
    vr.default_pagesize = pagesize



from voeventdb.remote.definitions import ResultKeys
from voeventdb.remote.apiv1.definitions import Endpoints, FilterKeys
from voeventdb.remote.request_wrappers import (
    get_summary_data, get_list_data, get_detail_response
)
import logging
from voeventdb.remote.helpers import _map_citations

logger = logging.getLogger(__name__)


def map_authored_month_count(filters=None,
                             host=None,
                             ):
    return get_summary_data(endpoint=Endpoints.map_authored_month_count,
                            filters=filters,
                            host=host)

def _fetch_refs(ivorn):
    return [r['ref_ivorn'] for r in packet_synopsis(ivorn)['refs']]

def _fetch_cites(ref_ivorn):
    return list_ivorn({FilterKeys.ref_exact : ref_ivorn})

def citation_network_map(ivorn, max_recursion_levels=5):
    defaultdict_map =  _map_citations(
        ivorn=ivorn,
        fetch_refs_func=_fetch_refs,
        fetch_citations_func=_fetch_cites,
        max_recursion=max_recursion_levels
    )
    return dict(defaultdict_map)


def count(filters=None,
          host=None,
          ):
    return get_summary_data(endpoint=Endpoints.count,
                            filters=filters,
                            host=host)


def list_ivorn(filters=None,
               order=None,
               pagesize=None,
               n_max=None,
               host=None,
               ):
    return get_list_data(list_endpoint=Endpoints.list_ivorn,
                         count_endpoint=Endpoints.count,
                         filters=filters,
                         order=order,
                         pagesize=pagesize,
                         n_max=n_max,
                         host=host,
                         )


def list_ivorn_ncites(filters=None,
                      order=None,
                      pagesize=None,
                      n_max=None,
                      host=None,
                      ):
    return get_list_data(list_endpoint=Endpoints.list_ivorn_ncites,
                         count_endpoint=Endpoints.count,
                         filters=filters,
                         order=order,
                         pagesize=pagesize,
                         n_max=n_max,
                         host=host,
                         )


def list_ivorn_nrefs(filters=None,
                     order=None,
                     pagesize=None,
                     n_max=None,
                     host=None,
                     ):
    return get_list_data(list_endpoint=Endpoints.list_ivorn_nrefs,
                         count_endpoint=Endpoints.count,
                         filters=filters,
                         order=order,
                         pagesize=pagesize,
                         n_max=n_max,
                         host=host,
                         )


def map_role_count(filters=None,
                   host=None,
                   ):
    return get_summary_data(endpoint=Endpoints.map_role_count,
                            filters=filters,
                            host=host)


def map_stream_count(filters=None,
                     host=None,
                     ):
    return get_summary_data(endpoint=Endpoints.map_stream_count,
                            filters=filters,
                            host=host)


def map_stream_role_count(filters=None,
                          host=None,
                          ):
    return get_summary_data(endpoint=Endpoints.map_stream_role_count,
                            filters=filters,
                            host=host)


def packet_synopsis(ivorn,
                    host=None):
    r = get_detail_response(endpoint=Endpoints.packet_synopsis,
                            ivorn=ivorn,
                            host=host)
    return r.json()[ResultKeys.result]


def packet_xml(ivorn,
               host=None):
    r = get_detail_response(endpoint=Endpoints.packet_xml,
                            ivorn=ivorn,
                            host=host)
    return r.text.encode('utf-8')



from contextlib import contextmanager
import requests
import logging
logger = logging.getLogger(__name__)

@contextmanager
def helpful_requests_error_log():
    """
    Provide 'ERROR' level log-messages pointing to common user-errors.

    Requests exceptions can be a bit cryptic for the uninitiated,
    so this context tries to give some helpful clues.

    """
    try:
        yield
    except requests.exceptions.ConnectionError:
        logger.error(
            "Could not establish connection, double-check your host address?")
        raise
    except requests.exceptions.MissingSchema:
        logger.error(
            "Could not establish connection - "
            "did you forget the 'http://' prefix when specifying the host?")
        raise
    finally:
        pass




"""
Classes enumerated things specific to apiv1.

These are intended to be useful to the end-user in building queries.
"""

class Endpoints:
    ## Regex from server inspection output:
    ## ^'(.*)': (.*').* -> $1 = $2
    ## Then:
    ## ([\w]*)/(.*) -> $1_$2
    root = '/apiv1/'
    count = '/apiv1/count'
    list_ivorn = '/apiv1/list/ivorn'
    list_ivorn_ncites = '/apiv1/list/ivorn_ncites'
    list_ivorn_nrefs = '/apiv1/list/ivorn_nrefs'
    map_authored_month_count = '/apiv1/map/authored_month_count'
    map_role_count = '/apiv1/map/role_count'
    map_stream_count = '/apiv1/map/stream_count'
    map_stream_role_count = '/apiv1/map/stream_role_count'
    packet_synopsis = '/apiv1/packet/synopsis/'
    packet_xml = '/apiv1/packet/xml/'


class FilterKeys:
    """
    Enumerates valid filters.

    Useful for building dictionaries representing filter-sets, eg.::

        filters = {
            FilterKeys.ivorn_contains: 'GRB',
            FilterKeys.role : 'observation',
        }

    For definitions of the various filters, and examples of valid
    values, see the :ref:`voeventdb query-filters <voeventdb:apiv1_filters>` page.

    """
    authored_since = 'authored_since'
    authored_until = 'authored_until'
    cited = 'cited'
    cone = 'cone'
    coord = 'coord'
    ivorn_contains = 'ivorn_contains'
    ivorn_prefix = 'ivorn_prefix'
    ref_any = 'ref_any'
    ref_contains = 'ref_contains'
    ref_exact = 'ref_exact'
    role = 'role'
    stream = 'stream'

class OrderValues:
    """
    Enumerates valid orderings.

    For details see :ref:`this section <voeventdb:ordervalues>` in
    the *voeventdb.server* docs.

    """
    author_datetime = 'author_datetime'
    author_datetime_desc = '-author_datetime'
    id = 'id'
    id_desc = '-id'
    ivorn = 'ivorn'
    ivorn_desc = '-ivorn'






"""
Helper classes for providing convenience parsing of returned JSON content.
"""
from astropy.coordinates import SkyCoord, Angle
from astropy.time import Time
import iso8601
import logging
from copy import deepcopy
from collections import defaultdict

logging.getLogger('iso8601').setLevel(logging.INFO)
import pprint

logger = logging.getLogger(__name__)


class SkyEvent(object):
    """
    Represents universal attributes of an observed event on sky.

    I.e. the most basic details that we expect to find in all packets
    reporting events with sky-positions.

    Attributes:
        position (:class:`astropy.coordinates.SkyCoord`): Best-estimate
            sky-coordinates of the event being reported.

        position error (:class:`astropy.coordinates.Angle`): Error-cone on the
            position estimate.

        timestamp of event (:class:`datetime.datetime`):
            Timestamp for the reported event (UTC timezone).

    """

    def __init__(self, skyevent_dict):
        d = skyevent_dict
        self.position = SkyCoord(d['ra'], d['dec'], unit='deg')
        self.position_error = Angle(d['error'], unit='deg')
        self.timestamp = None
        if d.get('time'):
            self.timestamp = iso8601.parse_date(d['time'])

    def __repr__(self):
        return (str(self.position)
                + ' +/- ' + self.position_error.to_string(decimal=True)
                + ' @ ' + self.timestamp.isoformat())

    @staticmethod
    def _parse_coords_dict(coords_dict):
        d = coords_dict
        posn = SkyCoord(d['ra'], d['dec'], unit='deg')
        posn_error = Angle(d['error'], unit='deg')
        time = iso8601.parse_date(d['time'])
        return SkyEvent(sky_position=posn,
                        sky_position_error=posn_error,
                        event_timestamp=time)


class Synopsis(object):
    """
    Parses the output from the 'synopsis' endpoint into a class-object.

    Attributes:
        author_datetime (:class:`datetime.datetime`): The ``Who.Date`` timestamp.
            (Parsed to datetime, UTC timezone). May be ``None`` if no timestamp
            present in the VOEvent packet.
        author_ivorn (string): The ``Who.AuthorIVORN`` entry. May be
            None if no entry present in the VOEvent packet.
        ivorn (string)
        received (:class:`datetime.datetime`): Dates exactly when the packet
            was loaded into this instance of voeventdb.
        role (string)
        stream (string)
        version (string)
        sky_events (list): A list of :class:`SkyEvent` objects.
            These are parsed from the `WhereWhen` section of a VOEvent
            packet, if possible. (If the packet has no WhereWhen data,
            this is an empty list).
        coords (list): :class:`astropy.coordinates.SkyCoord` positions.
            This is a syntax-shortcut, the positions are just those of the
            :attr:`.sky_events` entries, if any are present.
        references (list): References to other VOEvent packets.
            This is a list of dictionaries representing any references
            present in the ``Citations`` section of the VOEvent packet.

    """

    def __init__(self, synopsis_dict, api_version_string='apiv1'):

        voevent_dict = synopsis_dict['voevent']

        self.author_datetime = None
        if voevent_dict['author_datetime']:
            self.author_datetime = iso8601.parse_date(
                voevent_dict['author_datetime'])

        self.author_ivorn = None
        if voevent_dict['author_ivorn']:
            self.author_ivorn = voevent_dict['author_ivorn']

        self.ivorn = voevent_dict['ivorn']
        self.received = iso8601.parse_date(
            voevent_dict['received'])
        self.role = voevent_dict['role']
        self.stream = voevent_dict['stream']
        self.version = voevent_dict['version']

        self._synopsis_dict = synopsis_dict.copy()

        self.sky_events = [SkyEvent(d) for d in synopsis_dict['coords']]

        self.coords = [e.position for e in self.sky_events]

        self.references = self._synopsis_dict['refs']

    def __repr__(self):
        # Can be pasted back in to recreate the object
        return "Synopsis({})".format(self._synopsis_dict)

    def __str__(self):
        # Bit hacky, probably overkill:
        display_vars = {k: v for k, v in vars(self).items()
                        if not k.startswith('_')}
        display_vars = deepcopy(display_vars)
        display_vars.pop('coords')
        for k, v in display_vars.items():
            if hasattr(v, 'isoformat'):
                display_vars[k] = v.isoformat()

        pp = pprint.PrettyPrinter(indent=2)
        return pp.pformat(display_vars)


def _map_citations(ivorn,
                   fetch_refs_func,
                   fetch_citations_func,
                   cite_map=None,
                   previously_mapped=None,
                   prev_recursion_level=None,
                   max_recursion=None,
                   ):
    """
    Perform a depth first search on the citation-network.

    (API agnostic version, should be wrapped for convenience)

    Stop at max_recursion links from initial node / IVORN.

    Returns:
        citation_map (dict): A dictionary containing all the VOEvents in this
            network, to max_recursion level specified.
            The dictionary maps an 'origin' IVORN to its citing packets,
            i.e.::

                { origin_ivorn : {citing_ivorn1, ..., citing_ivornN} }

            We could have inverted the relationship (map packet_ivorn->included references)
            but since the typical behaviour is for many packets to cite one origin
            packet, this usually results in a more compact representation.

    """

    if prev_recursion_level is None:
        current_recursion_level = 0
    else:
        current_recursion_level = prev_recursion_level + 1
    logger.debug('Fetching for {}, at recursion level {}'.format(
        ivorn, current_recursion_level))

    if cite_map is None:
        cite_map = defaultdict(set)
    if previously_mapped is None:
        previously_mapped = set()
    logger.debug('{} packets mapped'.format(len(previously_mapped)))
    ref_ivorns = fetch_refs_func(ivorn)
    for ri in ref_ivorns:
        # Add a 'citation' entry to the cite_map of the origin-ivorn being referenced.
        cite_map[ri].add(ivorn)
    citing_ivorns = fetch_citations_func(ivorn)
    for ci in citing_ivorns:
        cite_map[ivorn].add(ci)
    previously_mapped.add(ivorn)
    if not max_recursion or (current_recursion_level < max_recursion):
        for ri in ref_ivorns:
            if ri not in previously_mapped:
                cite_map.update(_map_citations(
                    ivorn=ri,
                    fetch_refs_func=fetch_refs_func,
                    fetch_citations_func=fetch_citations_func,
                    cite_map=cite_map,
                    previously_mapped=previously_mapped,
                    prev_recursion_level=current_recursion_level,
                    max_recursion=max_recursion,
                ))
        for ci in citing_ivorns:
            if ci not in previously_mapped:
                cite_map.update(_map_citations(
                    ivorn=ci,
                    fetch_refs_func=fetch_refs_func,
                    fetch_citations_func=fetch_citations_func,
                    cite_map=cite_map,
                    previously_mapped=previously_mapped,
                    prev_recursion_level=current_recursion_level,
                    max_recursion=max_recursion,
                ))
    return cite_map



"""
Define the keys / values used in querystrings that are same across API-versions.
"""

class PaginationKeys:
    limit = 'limit'
    offset = 'offset'
    order = 'order'

class ResultKeys:
    endpoint = 'endpoint'
    limit = 'limit'
    querystring = 'querystring'
    result = 'result'
    url = 'url'




from __future__ import absolute_import
from voeventdb.remote.apiv1.definitions import (
    Endpoints, FilterKeys, OrderValues
)
from voeventdb.remote.apiv1.convenience_funcs import (
    map_authored_month_count,
    citation_network_map,
    count,
    list_ivorn,
    list_ivorn_ncites,
    list_ivorn_nrefs,
    map_role_count,
    map_stream_count,
    map_stream_role_count,
    packet_synopsis,
    packet_xml,
)




from __future__ import print_function
import pytest
import voeventdb.server.restapi.inspection_utils as iu
from voeventdb.server.tests.resources import swift_bat_grb_655721
from voeventdb.server.restapi.v1.definitions import _list_class_vars
import voeventdb.remote as vr
import voeventdb.remote.apiv1 as apiv1
import voeventdb.remote.apiv1.definitions
import voeventparse as vp
import requests
import logging
logger = logging.getLogger(__name__)

from voeventdb.remote.helpers import Synopsis

def test_endpoint_urls_in_sync():
    v1_ep_upstream = iu.apiv1_endpoints().values()
    v1_ep_remote = _list_class_vars(vr.apiv1.definitions.Endpoints).values()
    assert  sorted(v1_ep_upstream) == sorted(v1_ep_remote)


@pytest.mark.usefixtures('mock_requests')
class TestFunctionCalls():
    def test_ordering_param(self, simple_populated_db):
        """Just a sanity check that ordering syntax is correct"""
        dbinf = simple_populated_db
        all_ivorns = apiv1.list_ivorn(order=apiv1.OrderValues.id)
        all_ivorns_rev_order = apiv1.list_ivorn(order=apiv1.OrderValues.id_desc)
        assert all_ivorns == dbinf.inserted_ivorns
        assert all_ivorns_rev_order == all_ivorns[::-1]

    def test_list_pagination(self, simple_populated_db,
                            reset_globals_to_defaults):
        """
        Sanity check that pagination params passed correctly to wrapper func.

        (NB wrapper tested independently elsewhere).
        """
        vr.default_list_n_max = 30
        vr.default_pagesize = 5
        dbinf = simple_populated_db
        count = apiv1.count()
        assert count > vr.default_list_n_max
        ivorns = apiv1.list_ivorn()
        assert len(ivorns) == vr.default_list_n_max
        #This time we override the max:
        ivorns = apiv1.list_ivorn(n_max=0)
        assert len(ivorns) == dbinf.n_inserts

    def test_xml_retrieval(self, simple_populated_db):
        dbinf = simple_populated_db
        all_ivorns = apiv1.list_ivorn(order=apiv1.OrderValues.id)
        xml = apiv1.packet_xml(all_ivorns[-1])
        assert xml == vp.dumps(dbinf.insert_packets[-1])

        #Now try a non-existent ivorn:
        with pytest.raises(requests.HTTPError):
            apiv1.packet_xml('ivo://foo/bar')

    def test_synopsis(self, simple_populated_db):
        """Check synopsis wrapper function, helper class."""
        dbinf = simple_populated_db

        # Try an packet with references
        ivorns_w_refs = apiv1.list_ivorn(
            filters={apiv1.FilterKeys.ref_any: True},
            order = apiv1.OrderValues.id,
        )
        s = Synopsis(apiv1.packet_synopsis(ivorns_w_refs[0]))
        assert s.references
        assert len(s.sky_events) == 0

        #Now try a non-existent ivorn:
        with pytest.raises(requests.HTTPError):
            apiv1.packet_synopsis('ivo://foo/bar')

        # Now try a packet with co-ords
        sb_ivorn = swift_bat_grb_655721.attrib['ivorn']
        assert sb_ivorn in dbinf.inserted_ivorns
        s = Synopsis(apiv1.packet_synopsis(sb_ivorn))
        assert s.coords
        assert len(s.coords) == 1
        skyevent = s.sky_events[0]
        c_pkt = vp.pull_astro_coords(swift_bat_grb_655721)
        assert skyevent.position.ra.value == c_pkt.ra
        assert skyevent.position.dec.value == c_pkt.dec
        assert skyevent.position_error.value == c_pkt.err





from __future__ import print_function
import pytest
import inspect
import voeventdb.remote.apiv1 as apiv1
from voeventdb.remote.apiv1 import FilterKeys

import logging

logger = logging.getLogger(__name__)

apiv1_funcs_dict = {k: v for k, v in vars(apiv1).items()
                    if inspect.isfunction(v)
                    and not k.startswith('_')}

apiv1_queryfuncs_dict = {k:v for k,v in apiv1_funcs_dict.items()
                         if 'packet' not in k}
apiv1_queryfuncs_dict.pop('citation_network_map')


@pytest.mark.usefixtures('mock_requests')
@pytest.mark.parametrize("funcname,func",
                         tuple(apiv1_queryfuncs_dict.items())
                         )
class TestFunctionCalls():
    def test_queryfuncs(self, simple_populated_db,
                        funcname, func):
        logger.debug("Testing {}".format(funcname))

        # Test with default params:
        results = func()
        # print(results)

        # Test with a filter applied:
        filter = {FilterKeys.ivorn_prefix: 'ivo://',
                  FilterKeys.role: 'utility'}
        results = func(filters=filter)



import urllib
import requests
import requests.exceptions

import voeventdb.remote
from voeventdb.remote.definitions import (
    PaginationKeys,
    ResultKeys,
)
from voeventdb.remote.utils import (
    helpful_requests_error_log,
)
from astropy.coordinates import SkyCoord, Angle
from six import string_types
import datetime
import json
import logging

logger = logging.getLogger(__name__)

def format_conesearch_params(input):
    """
    Validates and formats parameters for the cone-search filter.
    """
    if not input:
        raise ValueError(
            "Cannot format {} to valid cone-search params".format(input()))

    try:
        if isinstance(input, string_types):
            # Assume preformatted (ra,dec,err) json-list:
            ra,dec,err = json.loads(input)
            posn = SkyCoord(ra,dec, unit='deg')
            posn_error = Angle(err, unit='deg')
        elif len(input)==2:
            # Assume (Skycoord, Angle) tuple:
            posn = SkyCoord(input[0])
            posn_error = Angle(input[1])
        elif len(input)==3:
            # Assume simple (ra,dec,err) tuple:
            # We still parse via SkyCoord, this provides validation.
            posn = SkyCoord(input[0],input[1], unit='deg')
            posn_error = Angle(input[2], unit='deg')
        else:
            raise ValueError
    except:
        logger.error(
            "Cannot format {} to valid SkyCoord / Angle "
            "for cone-search ".format(input()))
        raise
    return str([posn.ra.deg, posn.dec.deg, posn_error.deg])


def format_filters(filters):
    """
    Perform any necessary autoconversion of filter-values to strings.

    For example, we isoformat any datetimes.
    """
    if not filters:
        return filters
    formatted = {}
    for k,v in filters.items():
        if isinstance(v, datetime.datetime):
            formatted[k] = v.isoformat()
        else:
            formatted[k]=v
    if 'cone' in filters:
        formatted['cone'] = format_conesearch_params(filters['cone'])
    return formatted

def get_summary_response(endpoint,
                         filters,
                         host,
                         ):
    if host is None:
        host = voeventdb.remote.default_host
    params = format_filters(filters)
    with helpful_requests_error_log():
        r = requests.get(host + endpoint,
                         params=params
                         )
    r.raise_for_status()
    return r


def get_summary_data(endpoint,
                     filters,
                     host,
                     ):
    r = get_summary_response(endpoint, filters, host)
    return r.json()[ResultKeys.result]


def get_detail_response(endpoint,
                        ivorn,
                        host):
    if host is None:
        host = voeventdb.remote.default_host
    ep_url = host + endpoint
    url = ep_url + urllib.quote_plus(ivorn)
    with helpful_requests_error_log():
        r = requests.get(url)
    r.raise_for_status()
    return r

def get_paginated(url, params, n_to_fetch, pagesize):
    offset = 0
    results = []
    localpars = params.copy()
    localpars[PaginationKeys.limit] = pagesize
    if n_to_fetch < pagesize:
        localpars[PaginationKeys.limit] = n_to_fetch
    while offset < n_to_fetch:
        localpars[PaginationKeys.offset] = offset
        with helpful_requests_error_log():
            r = requests.get(url,
                             params=localpars
                             )
        r.raise_for_status()
        results.extend(r.json()[ResultKeys.result])
        offset += pagesize
    # Snip any excess results caused by n_to_fetch not being
    # a multiple of pagesize
    results = results[:n_to_fetch]
    return results

def get_list_data(list_endpoint,
                  count_endpoint,
                  filters=None,
                  order=None,
                  n_max=None,
                  pagesize=None,
                  host=None,
                  ):
    if host is None:
        host = voeventdb.remote.default_host
    if pagesize is None:
        pagesize = voeventdb.remote.default_pagesize
    params = {}
    if filters:
        params.update(format_filters(filters))

    n_matched = get_summary_data(endpoint=count_endpoint,
                                filters=filters,
                                host=host)
    if n_max is None:
        n_max = voeventdb.remote.default_list_n_max
    if n_max:
        n_to_fetch = min(n_matched, n_max)
    else:
        n_to_fetch = n_matched

    if order:
        params[PaginationKeys.order] = order

    results = get_paginated(url=host + list_endpoint,
                            params=params,
                            n_to_fetch=n_to_fetch,
                            pagesize=pagesize,
                            )

    return results



#!/usr/bin/env python

from __future__ import print_function
from setuptools import setup, find_packages
import versioneer


install_requires = [
    'iso8601',
    'pytz',
    'requests',
    'simplejson',
    'astropy',
    'six',
]

test_requires = [
    'voeventdb.server[test]',
]

extras_require = {
    'test': test_requires,
    'all': test_requires,
}
packages = find_packages()
print("FOUND PACKAGES: ", packages)


setup(
    name="voeventdb.remote",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="Client-library for remotely querying the voeventdb REST API.",
    author="Tim Staley",
    author_email="timstaley337@gmail.com",
    url="https://github.com/timstaley/voeventdb.remote",
    packages=packages,
    install_requires=install_requires,
    extras_require=extras_require,
)



"""
Define some package-level default values:
"""
from __future__ import absolute_import

from ._version import get_versions
__versiondict__ = get_versions()
__version__ = __versiondict__['version']
del get_versions

import voeventdb.remote.definitions

default_host = 'http://voeventdb.4pisky.org'
"""
The default host to query.
"""

default_list_n_max = 5000
"""
Maximum number of rows to fetch from a list request, by default.

If you really want to fetch more rows in a single query, this can be
changed for a single request by setting the ``n_max`` parameter-value.
Setting a value of ``n_max=0`` will fetch all available rows.
"""


default_pagesize = 1000
"""
Number of rows fetched in each HTTP GET request (when querying list-endpoints).

Note that if more rows are available, multiple GET requests are
made 'behind the scenes' when using the top-level interface,
so this variable can typically be left unchanged
for casual usage - it's aimed at advanced usage when trying to improve
performance with large queries.
"""

